# encoding: utf-8
import logging

import pwe.lib.validators as pwe_validators
from efa.lib.validators import (
    euhu_validator_factory,
    build_data_to_validate
)
from efa.model.participation import Participation
from efa.views.participation import (
    update_ as efa_update,
    create_ as efa_create,
    generate_controllist)
from euhu.programs.pwe import validate
from pwe.model.participation import PWEParticipation
from pwe.views.participant import extract_parts
from pyramid.httpexceptions import HTTPUnauthorized
from pyramid.view import view_config
from ringo.lib import security
from ringo.lib.helpers import get_action_routename
from ringo.views.helpers import (
    get_item_from_request)
from efa.model.project import Project

log = logging.getLogger(__name__)


DEFAULT_FP2_PROJECTID = 2


def euhu_validator(field, data, request):
    participation = get_item_from_request(request)
    # For GET request return the cached validation state.
    if request.method == "GET" and hasattr(participation, "valid"):
        if participation.valid is not None:
            return participation.valid

    # Revalidate the participation
    data_factory = build_data_to_validate(data, participation)
    data_factory['gender'] = (request.POST.get('participant.gender') or
                              data_factory['gender'])
    # Override persisted gender with current value for validation. See issue2921
    # Save the result of the validation.
    if hasattr(participation, "valid"):
        if validate(data_factory).has_violated_rules():
            participation.valid = 0
            return False
        else:
            participation.valid = 1
            return True
    return True


def create_callback(request, item):
    if request.session.get("participation_partial_oi"):
        partial_oi = request.session.get("participation_partial_oi")
    if request.session.get("participant_partial_oi"):
        partial_oi = request.session.get("participant_partial_oi")
    if partial_oi:
        for k, v in partial_oi.iteritems():
            setattr(item.pweoi, k, v)
        request.session['participant_partial_oi'] = None
        request.session['participation_partial_oi'] = None
    return item


def add_hdl_integration(request, item):
    if item.pweri.hdl_integration_fp2:
        integration_sector_fp2_is_hdl = 26
        item.pweri.integration_sector_fp2 = integration_sector_fp2_is_hdl
    return item


@view_config(route_name=get_action_routename(Participation, 'create'),
             renderer='/default/create.mako',
             permission='create',
             request_method="POST")
def create_(request):
    request.session['participation_partial_oi'] = extract_parts(request)
    validator = pwe_validators.get_eligibility_validators(request)
    if request.params.get('_project'):
        project_id = int(request.params.get('_project'))
        if project_id != request.POST['project']:
            request.POST['project'] = project_id
    elif request.session.get('participant_project_id'):
        project_id = request.session.get('participant_project_id')
    else:
        project_id = DEFAULT_FP2_PROJECTID
    project = request.db.query(Project).get(project_id)
    created = efa_create(request, external_validators=validator,
                         external_callbacks=[create_callback],
                         values={"project": project})
    return created


@view_config(route_name=get_action_routename(Participation, 'create'),
             renderer='/default/create.mako',
             permission='create',
             request_method="GET")
def create_GET(request):
    participant_project_id = request.session.get('participant_project_id')
    project = None
    if participant_project_id is not None:
        project = request.db.query(Project).get(participant_project_id)
    values = {}
    if project:
        values['project'] = project
    created = efa_create(request, values=values)
    return created


@view_config(route_name=get_action_routename(Participation, 'update'),
             renderer='/default/update.mako',
             permission='update')
def update_(request):
    validators = pwe_validators.get_eligibility_validators(request)
    validators.append(euhu_validator_factory(request, euhu_validator))
    return efa_update(request, validators, callbacks=[add_hdl_integration])


@view_config(route_name="controllist",
             renderer="/default/list.mako")
def controllist(request):
    """Display a summary list of participations for organisation users."""
    if not security.has_role(request.user, "beneficiary"):
        raise HTTPUnauthorized
    return generate_controllist(request, PWEParticipation)
